@@ -38,6 +38,14 @@ def custom_type(a):
     return a
 
 
+def build_formatter(output_file, **kwargs):
+    conf = cfg.ConfigOpts()
+    conf.register_opts(generator._generator_opts)
+    for k, v in kwargs.items():
+        conf.set_override(k, v)
+    return generator._OptFormatter(conf, output_file=output_file)
+
+
 class GeneratorTestCase(base.BaseTestCase):
 
     groups = {
@@ -1006,6 +1014,41 @@ class GeneratorTestCase(base.BaseTestCase):
             self.assertFalse(mock_log.warning.called)
 
 
+class GeneratorFileHandlingTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super(GeneratorFileHandlingTestCase, self).setUp()
+
+        self.conf = cfg.ConfigOpts()
+        self.config_fixture = config_fixture.Config(self.conf)
+        self.config = self.config_fixture.config
+
+    @mock.patch.object(generator, '_get_groups')
+    @mock.patch.object(generator, '_list_opts')
+    def test_close_generated_file(self, a, b):
+        generator.register_cli_opts(self.conf)
+        self.config(output_file='somefile')
+
+        m = mock.mock_open()
+        m.close = mock.Mock()
+
+        with mock.patch.object(generator, 'open', m, create=True):
+            generator.generate(self.conf, output_file=None)
+
+        m().close.assert_called_once()
+
+    @mock.patch.object(generator, '_get_groups')
+    @mock.patch.object(generator, '_list_opts')
+    def test_not_close_external_file(self, a, b):
+        generator.register_cli_opts(self.conf)
+        self.config()
+
+        m = mock.Mock()
+        generator.generate(self.conf, output_file=m)
+
+        m().close.assert_not_called()
+
+
 class DriverOptionTestCase(base.BaseTestCase):
 
     def setUp(self):
@@ -1085,7 +1128,8 @@ GENERATOR_OPTS = {'format_': 'yaml',
                   'namespace': ['test'],
                   'output_file': None,
                   'summarize': False,
-                  'wrap_width': 70}
+                  'wrap_width': 70,
+                  'config_source': []}
 
 
 class MachineReadableGeneratorTestCase(base.BaseTestCase):
@@ -1497,7 +1541,7 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
 
         fd, tmp_file = tempfile.mkstemp()
         with open(tmp_file, 'w+') as f:
-            formatter = generator._OptFormatter(output_file=f)
+            formatter = build_formatter(f)
             generator._output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
         expected = '''[DEFAULT]
 '''
@@ -1513,7 +1557,7 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
 
         fd, tmp_file = tempfile.mkstemp()
         with open(tmp_file, 'w+') as f:
-            formatter = generator._OptFormatter(output_file=f)
+            formatter = build_formatter(f)
             generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
         expected = '''[alpha]
 
@@ -1536,7 +1580,7 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
 
         fd, tmp_file = tempfile.mkstemp()
         f = open(tmp_file, 'w+')
-        formatter = generator._OptFormatter(output_file=f)
+        formatter = build_formatter(f)
         expected = '''[alpha]
 
 #
@@ -1563,7 +1607,7 @@ class GeneratorMutableOptionTestCase(base.BaseTestCase):
     def test_include_message(self):
         out = moves.StringIO()
         opt = cfg.StrOpt('foo', help='foo option', mutable=True)
-        gen = generator._OptFormatter(output_file=out)
+        gen = build_formatter(out)
         gen.format(opt, 'group1')
         result = out.getvalue()
         self.assertIn(
@@ -1574,7 +1618,7 @@ class GeneratorMutableOptionTestCase(base.BaseTestCase):
     def test_do_not_include_message(self):
         out = moves.StringIO()
         opt = cfg.StrOpt('foo', help='foo option', mutable=False)
-        gen = generator._OptFormatter(output_file=out)
+        gen = build_formatter(out)
         gen.format(opt, 'group1')
         result = out.getvalue()
         self.assertNotIn(
@@ -1668,11 +1712,10 @@ class RequiredOptionTestCase(base.BaseTestCase):
 
         fd, tmp_file = tempfile.mkstemp()
         with open(tmp_file, 'w+') as f:
-            formatter = generator._OptFormatter(output_file=f)
+            formatter = build_formatter(f, minimal=True)
             generator._output_opts(formatter,
                                    'alpha',
-                                   groups.pop('alpha'),
-                                   minimal=True)
+                                   groups.pop('alpha'))
         expected = '''[alpha]
 
 #
@@ -1726,11 +1769,10 @@ messages!""")]
 
         fd, tmp_file = tempfile.mkstemp()
         with open(tmp_file, 'w+') as f:
-            formatter = generator._OptFormatter(output_file=f)
+            formatter = build_formatter(f, summarize=True)
             generator._output_opts(formatter,
                                    'alpha',
-                                   groups.pop('alpha'),
-                                   summarize=True)
+                                   groups.pop('alpha'))
         expected = '''[alpha]
 
 #
@@ -1768,7 +1810,7 @@ class AdvancedOptionsTestCase(base.BaseTestCase):
 
         fd, tmp_file = tempfile.mkstemp()
         with open(tmp_file, 'w+') as f:
-            formatter = generator._OptFormatter(output_file=f)
+            formatter = build_formatter(f)
             generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
         expected = '''[alpha]
 
@@ -1809,7 +1851,7 @@ class HostAddressTestCase(base.BaseTestCase):
         groups = generator._get_groups(config)
 
         out = moves.StringIO()
-        formatter = generator._OptFormatter(output_file=out)
+        formatter = build_formatter(out)
         generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
         result = out.getvalue()
 
